// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/18AO8/18AO8_Linux_1.x_and_2.x_DN/driver/main.h $
// $Rev: 52875 $
// $Date: 2023-04-18 08:57:44 -0500 (Tue, 18 Apr 2023) $

// 18AO8: Device Driver: header file

#ifndef	__MAIN_H__
#define	__MAIN_H__

#include "18ao8.h"

#define	DEV_BAR_SHOW						0
#define	DEV_METRICS_SHOW					0
#define	DEV_PCI_ID_SHOW						0
#define	DEV_SUPPORTS_IRQ
#define	DEV_SUPPORTS_WAIT
#define	DEV_SUPPORTS_WRITE
#define	GSC_ALT_DEV_GET(a)					(a)
#define	GSC_ALT_STRUCT_GET(i,d)				(d)
#define	GSC_ALT_STRUCT_T					dev_data_t
#define	GSC_DEVS_PER_BOARD					1

typedef struct _dev_data_t					dev_data_t;

#include "gsc_main.h"



// macros *********************************************************************

#define	DEV_MODEL							"18AO8"	// Upper case form of the below.
#define	DEV_NAME							"18ao8"	// MUST AGREE WITH AO8_BASE_NAME

#define	DEV_VERSION							"2.4"	// FOR DEVICE SPECIFIC CODE ONLY!
// 2.4	Updated to support the 6.x series kernel.
//		Updated comments for consistency.
//		Updated the initialization IOCTl service for consistency.
//		Updated the autocalibration IOCTL service for consistency.
//		Updated per PLX code segregation.
// 2.3	Updated initial file content for consistency.
//		Updated for porting to Windows.
//		Standardized various IOCTL system log messages.
//		Standardized processing of the initialization IOCTL service.
//		Standardized processing of the autocalibration IOCTL service.
//		LINTed source files.
// 2.2	Updated per changes to common code.
//		Added support for the 5.x kernel series.
//		Added support for Big Endian hosts.
//		Updated BMDMA macro spelling.
//		Remove Demand Mode DMA write code as DMDMA is not supported.
//		White space cleanup.
//		Bug fix: was ignoring BAR setup return value.
//		Updated per changes to the OS specific PCI services.
//		Made updates for DEV_SUPPORTS_XXX macro changes.
//		Modified code so open succeeds even if initialization fails.
// 2.1	The BAR code has been updated to include the data under a single structure.
//		The register definitions have been updated.
//		I/O services now pass around an os_mem_t structure instead of a buffer pointer.
//		White space cleanup.
//		Changed the arbitrary wait event callback argument to a void* type.// 2.0	Updated to use the newer common driver sources.
//		Modified to allow multiple apps to simultaneously access a single device.
//		Correct access to BAR0 and BAR1 (is RO, was RW).
//		Added an infinite I/O timeout option.
// 2.0	Updated to use the newer common driver sources.
//		Removed GNU notice from non-Linux specific files.
//		Removed Linux specific content from non-Linux specific source files.
//		White space cleanup.
//		Now using a spinlock rather than enabling and disabling interrupts.
//		Updated gsc_irq_open() and gsc_irq_close().
//		Updated gsc_dma_open() and gsc_dma_close().
// 1.2	Reduced #include list in driver interface header.
// 1.1	Corrected system timer count roll over bug in ioctl.c.
// 1.0	Initial release.

// I/O services
#define	DEV_IO_STREAM_QTY					(DEV_IO_RX_STREAM_QTY + DEV_IO_TX_STREAM_QTY)
#define	DEV_IO_RX_STREAM_QTY				0
#define	DEV_IO_TX_STREAM_QTY				1

#define	GSC_WRITE_PIO_WORK_32_BIT

// WAIT services
#define	DEV_WAIT_GSC_ALL					AO8_WAIT_GSC_ALL
#define	DEV_WAIT_ALT_ALL					AO8_WAIT_ALT_ALL
#define	DEV_WAIT_IO_ALL						AO8_WAIT_IO_ALL



// data types *****************************************************************

struct _dev_io_t
{
	// Initialized by open and IOCTL services (initialize and/or service specific).

	s32					dma_threshold;		// Minimum DMA size, if request not yet met.
	s32					io_mode;			// PIO, BMDMA, DMDMA
	s32					overflow_data;		// Check data overflow when writing?
	s32					overflow_frame;		// Check frame overflow when writing?
	s32					pio_threshold;		// Use PIO if samples <= this.
	s32					timeout_s;			// I/O timeout in seconds.

	// Initialized by I/O service at time of use.

	int					abort;
	gsc_dma_ch_t*		dma_channel;		// Use this channel for DMA.
	int					non_blocking;		// Is this non-blocking I/O?

	// Initialized by device specific I/O create code.

	int					bytes_per_sample;	// Sample size in bytes.
	u32					io_reg_offset;		// Offset of device's I/O FIFO.
	VADDR_T				io_reg_vaddr;		// Address of device's I/O FIFO.

	void				(*dev_io_sw_init)		(GSC_ALT_STRUCT_T* alt, dev_io_t* io);
	void				(*dev_io_close)			(GSC_ALT_STRUCT_T* alt, dev_io_t* io);
	void				(*dev_io_open)			(GSC_ALT_STRUCT_T* alt, dev_io_t* io);
	int					(*dev_io_startup)		(GSC_ALT_STRUCT_T* alt, dev_io_t* io);
	long				(*dev_pio_available)	(GSC_ALT_STRUCT_T* alt, dev_io_t* io, size_t count);
	long				(*dev_bmdma_available)	(GSC_ALT_STRUCT_T* alt, dev_io_t* io, size_t count);
	long				(*dev_dmdma_available)	(GSC_ALT_STRUCT_T* alt, dev_io_t* io, size_t count);
	long				(*dev_pio_xfer)			(GSC_ALT_STRUCT_T* alt, dev_io_t* io, const os_mem_t* mem, size_t count, os_time_tick_t st_end);
	long				(*dev_bmdma_xfer)		(GSC_ALT_STRUCT_T* alt, dev_io_t* io, const os_mem_t* mem, size_t count, os_time_tick_t st_end);
	long				(*dev_dmdma_xfer)		(GSC_ALT_STRUCT_T* alt, dev_io_t* io, const os_mem_t* mem, size_t count, os_time_tick_t st_end);

	struct
	{
		u32				abort;		// ...WAIT_IO_XXX_ABORT
		u32				done;		// ...WAIT_IO_XXX_DONE
		u32				error;		// ...WAIT_IO_XXX_ERROR
		u32				timeout;	// ...WAIT_IO_XXX_TIMEOUT
	} wait;

	// Initialize by GSC commone create code.

	os_mem_t			mem;				// I/O buffer.
	os_sem_t			sem;				// Only one Tx or Rx at a time.
};

struct _dev_data_t
{
	os_pci_t			pci;			// The kernel PCI device descriptor.
	os_data_t			os;				// OS specific data.
	os_spinlock_t		spinlock;		// Control ISR access.
	os_sem_t			sem;			// Control thread access.
	gsc_dev_type_t		board_type;		// Corresponds to basic device type.
	const char*			model;			// Base model number in upper case.
	int					board_index;	// Index of the device being accessed.
	int					users;			// Number of currently active open() requests.
	int					share;			// Were we opened in shared mode?

	gsc_bar_t			bar;			// device register mappings
	gsc_dma_t			dma;			// For DMA based I/O.
	gsc_irq_t			irq;			// For interrut support.
	gsc_wait_node_t*	wait_list;

	struct
	{					// This is for streaming I/O
		dev_io_t		tx;				// Analog Output write
		dev_io_t*		io_streams[DEV_IO_STREAM_QTY];
	} io;

	struct
	{
		VADDR_T			plx_intcsr_32;	// Interrupt Control/Status Register
		VADDR_T			plx_dmaarb_32;	// DMA Arbitration Register
		VADDR_T			plx_dmathr_32;	// DMA Threshold Register

		VADDR_T			gsc_asiocr_32;	// Auxiliary Sync I/O Control Register
		VADDR_T			gsc_bcr_32;		// Board Control Register
		VADDR_T			gsc_diopr_32;	// Digital I/O Port Register
		VADDR_T			gsc_psr_32;		// Primary Status Register
		VADDR_T			gsc_acr_32;		// Assembly Configuration Register
		VADDR_T			gsc_boor_32;	// Buffered Output Operations Register
		VADDR_T			gsc_obtr_32;	// Output Buffer Threshold Register
		VADDR_T			gsc_obsr_32;	// Output Buffer Size Register
		VADDR_T			gsc_aobr_32;	// Output Output Buffer Register
		VADDR_T			gsc_rcgr_32;	// Rate-C Generator Register
		VADDR_T			gsc_rdgr_32;	// Rate-D Generator Register
		VADDR_T			gsc_ocr_32;		// Output Configuration Register
		VADDR_T			gsc_scr_32;		// Selftest Configuration Register
		VADDR_T			gsc_sdbr_32;	// Selftest Data Buffer Register
	} vaddr;

	struct
	{
		u32				gsc_acr_32;			// Assembly Configuration Register

		s32				auto_cal_ms;		// Maximum ms for auto-cal

		s32				channel_mask;		// The channel selection mask.
		s32				channel_qty;		// The number of channels on the device.
		s32				channels_max;		// Maximum channels supported by model.
		s32				custom_features;	// see AO8_CUSTOM_FEAT_XXX

		s32				fgen_max;			// Maximum rate generator rate.
		s32				fgen_min;			// Minimum rate generator rate.
		u32				fifo_size;			// Size of FIFO - not the fill level.
		s32				fref_default;		// Default reference clock rate.
		s32				fsamp_max;			// Maximum sample rate.
		s32				fsamp_min;			// Minimum sample rate.

		s32				initialize_ms;		// Maximum ms for initialize

		s32				ndiv_max;			// Maximum Ndiv value.
		s32				ndiv_min;			// Minimum Ndiv value.

		s32				selftest_ms;		// Selftest period in ms.

	} cache;
};



// prototypes *****************************************************************

void		dev_io_close(dev_data_t* dev);
int			dev_io_create(dev_data_t* dev);
void		dev_io_destroy(dev_data_t* dev);
int			dev_io_open(dev_data_t* dev);
dev_io_t*	dev_io_write_select(dev_data_t* dev, size_t count);
int			dev_irq_create(dev_data_t* dev);
void		dev_irq_destroy(dev_data_t* dev);
int			dev_write_create(dev_data_t* dev, dev_io_t* io);

int			initialize_ioctl(dev_data_t* dev, void* arg);



#endif
