// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/18AO8/18AO8_Linux_1.x_and_2.x_DN/samples/signals/perform.c $
// $Rev: 52866 $
// $Date: 2023-04-17 14:41:34 -0500 (Mon, 17 Apr 2023) $

// 18AO8: Sample Application: source file

#include "main.h"



// variables ******************************************************************

static	u32		_data[128 * 1024];
static	long	_qty;	// Number of samples to send per block.



//*****************************************************************************
static void _init_data(int chans, long rate, int seconds)
{
	u32*		dest;
	int			eof		= 0;
	int			i;
	long		period_len;	// The length of the pattern in samples.
	long		period_tot;	// Pattern samples total for all channels.
	long		period;		// Pattern length in x units.
	long		samp;
	float		samp_dat;	// Total sample size of the data buffer.
	float		samp_sec;	// Samples Per Second for all active channels
	float		samp_tot;	// Samples Total for all active channels
	const char*	units;

	gsc_label("Initializing Data");
	memset(_data, 0, sizeof(_data));
	samp_sec	= ((float) rate) * chans;
	samp_tot	= ((float) rate) * chans * seconds;
	samp_dat	= sizeof(_data) / 4;

	period_len	= (rate <= 100) ? rate				//      10-      100 ->    10-   100
				: (rate <= 1000) ? (rate / 2)		//     101-    1,000 ->    50-   500
				: (rate <= 10000) ? (rate / 5)		//   1,001-   10,000 ->   200- 2,000
				: (rate <= 100000) ? (rate / 10)	//  10,001-  100,000 -> 1,000-10,000
				: (rate / 100);						// 100,001-1,000,000 -> 1,000-10,000
	period_tot	= period_len * chans;
	period		= (long) (1000000000. / rate * period_len);
	units		= "ns";

	if ((period % 1000) == 0)
	{
		period	/= 1000;
		units	= "us";
	}

	if ((period % 1000) == 0)
	{
		period	/= 1000;
		units	= "ms";
	}

	if (samp_tot <= SIZEOF_ARRAY(_data))
	{
		// We can get the entire image in the buffer.
		_qty	= (long) samp_tot;
	}
	else if (samp_sec <= SIZEOF_ARRAY(_data))
	{
		// We can get one second of the image in the buffer.
		_qty	= (long) samp_sec;
	}
	else if (period_tot <= SIZEOF_ARRAY(_data))
	{
		// We can get one period of the image in the buffer.
		_qty	= period_tot;
	}
	else
	{
		// We can get only a fraction of a period of the image in the buffer.
		_qty	= (samp_dat / chans) * chans;
	}

	// Initialize the data buffer.
	// The data is encoded in OFFSET BINARY format.
	dest	= _data;;

	for (i = 0, samp = 0; i < _qty; i += chans, samp++)
	{
		samp	= (samp < period_len) ? samp : 0;

		// 1st channel: sawtooth wave
		*dest++	= 0xFFFFFL - (0xFFFFFL * samp / (period_len - 1));

		eof++;

		if (eof >= 20)
		{
			eof	= 0;
			dest[-1]	|= 0x4000;	// set the EOF Flag
		}
	}

	dest[-1]	|= 0x100000;
	printf("DONE  (pattern length: %ld samples, %ld %s)\n", period_len, period, units);
}



//*****************************************************************************
int perform_tests(const args_t* args)
{
	s32	fsamp	= 100000L;
	int	errs	= 0;
	s32	ndiv;

	//===============================================================
	gsc_label("Setup");
	printf("\n");
	gsc_label_level_inc();

	errs	+= ao8_initialize		(args->fd, -1, 1);
	errs	+= ao8_xcvr_output		(args->fd, -1, 1, AO8_XCVR_OUTPUT_ENABLE,	NULL);
	errs	+= ao8_xcvr_pol			(args->fd, -1, 1, AO8_XCVR_POL_DEFAULT,		NULL);
	errs	+= ao8_xcvr_type		(args->fd, -1, 1, AO8_XCVR_TYPE_TTL,		NULL);
	errs	+= ao8_channel_sel		(args->fd, -1, 1, 0x01,						NULL);

	_init_data(1, fsamp, 5);

	gsc_label("Writing Data");
	ao8_write(args->fd, _data, _qty * 4);
	printf("Done\n");

	errs	+= ao8_buf_mode			(args->fd, -1, 1, AO8_BUF_MODE_CIRC,		NULL);

	gsc_label_level_dec();

	//===============================================================
	gsc_label("Clock Output");
	printf("\n");
	gsc_label_level_inc();

	errs	+= ao8_fsamp_ao_compute	(args->fd, -1, 1, fsamp, &ndiv, NULL);
	errs	+= ao8_gen_c_ndiv		(args->fd, -1, 1, ndiv,					NULL);
	errs	+= ao8_clock_src		(args->fd, -1, 1, AO8_CLOCK_SRC_RCG,	NULL);
	errs	+= ao8_gen_c_enable		(args->fd, -1, 1, AO8_GEN_ENABLE_YES,	NULL);
	errs	+= ao8_clock_enable		(args->fd, -1, 1, AO8_CLOCK_ENABLE_YES,	NULL);
	errs	+= ao8_clock_ready		(args->fd, -1, 1,						NULL);

	gsc_label_level_dec();

	//===============================================================
	gsc_label("Trigger Output");
	printf("\n");
	gsc_label_level_inc();

	errs	+= ao8_gen_d_ndiv		(args->fd, -1, 1, ndiv * 5,					NULL);
	errs	+= ao8_burst_trig_src	(args->fd, -1, 1, AO8_BURST_TRIG_SRC_RDG,	NULL);
	errs	+= ao8_gen_d_enable		(args->fd, -1, 1, AO8_GEN_ENABLE_YES,		NULL);
	errs	+= ao8_burst_enable		(args->fd, -1, 1, AO8_BURST_ENABLE_YES,		NULL);
	errs	+= ao8_burst_ready		(args->fd, -1, 1,							NULL);

	gsc_label_level_dec();

	//===============================================================
	gsc_label("Waiting");
	printf("%d seconds ... ", args->seconds);
	fflush(stdout);
	os_sleep_ms(args->seconds * 1000);
	printf("Done\n");

	errs	+= ao8_initialize(args->fd, -1, 1);

	return(errs);
}



