// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/18AO8/18AO8_Linux_1.x_and_2.x_DN/utils/util_query.c $
// $Rev: 52859 $
// $Date: 2023-04-17 14:38:13 -0500 (Mon, 17 Apr 2023) $

// 18AO8: Utilities: source file

#include "main.h"



/******************************************************************************
*
*	Function:	ao8_query
*
*	Purpose:
*
*		Provide a visual wrapper for the AO8_IOCTL_QUERY service.
*
*	Arguments:
*
*		fd		Use this handle to access the device.
*
*		index	The index of the device to access. Ignore if < 0.
*				This is for display purposes only.
*
*		verbose	Work verbosely?
*
*		set		This is the query option to access.
*
*		get		The results are reported here. This may be NULL.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int ao8_query(int fd, int index, int verbose, s32 set, s32* get)
{
	char		buf[128];
	int			errs	= 0;
	const char*	ptr;
	s32			query	= set;
	int			ret;
	s32			tmp;

	switch (query)
	{
		default:

			errs++;
			ptr	= buf;
			sprintf(buf, "Query Error (#%ld)", (long) query);
			break;

		case AO8_QUERY_AUTO_CAL_MS:		ptr	= "Auto-Cal Duration";	break;
		case AO8_QUERY_CHANNEL_MASK:	ptr	= "Channel Mask";		break;
		case AO8_QUERY_CHANNEL_MAX:		ptr	= "Max Channels";		break;
		case AO8_QUERY_CHANNEL_QTY:		ptr	= "Installed Channels";	break;
		case AO8_QUERY_COUNT:			ptr	= "Query Options";		break;
		case AO8_QUERY_CUSTOM_FEAT:		ptr	= "Custom Feature";		break;
		case AO8_QUERY_DEVICE_TYPE:		ptr	= "Device Type";		break;
		case AO8_QUERY_FGEN_MAX:		ptr	= "Fgen  Max";			break;
		case AO8_QUERY_FGEN_MIN:		ptr	= "Fgen Min";			break;
		case AO8_QUERY_FIFO_SIZE:		ptr	= "FIFO Size";			break;
		case AO8_QUERY_FREF_DEFAULT:	ptr	= "Fref Default";		break;
		case AO8_QUERY_FSAMP_MAX:		ptr	= "Fsamp Max";			break;
		case AO8_QUERY_FSAMP_MIN:		ptr	= "Fsamp Min";			break;
		case AO8_QUERY_INIT_MS:			ptr	= "Init. Duration";		break;
		case AO8_QUERY_NDIV_MAX:		ptr	= "Ndiv Max";			break;
		case AO8_QUERY_NDIV_MIN:		ptr	= "Ndiv Min";			break;
		case AO8_QUERY_SELFTEST_MS:		ptr	= "Selftest Duration";	break;
	}

	if (verbose)
		gsc_label_index(ptr, index);

	ret		= ao8_ioctl(fd, AO8_IOCTL_QUERY, &set);
	errs	+= ret ? 1 : 0;

	switch (query)
	{
		default:

			errs++;
			sprintf(buf, "%ld", (long) set);
			break;

		case AO8_QUERY_CHANNEL_MAX:
		case AO8_QUERY_CHANNEL_QTY:
		case AO8_QUERY_COUNT:
		case AO8_QUERY_NDIV_MAX:
		case AO8_QUERY_NDIV_MIN:

			gsc_label_long_comma_buf(set, buf);
			break;

		case AO8_QUERY_CHANNEL_MASK:

			sprintf(buf, "0x%lX", (long) set);
			break;

		case AO8_QUERY_FGEN_MAX:
		case AO8_QUERY_FGEN_MIN:
		case AO8_QUERY_FREF_DEFAULT:

			gsc_label_long_comma_buf(set, buf);
			strcat(buf, " Hz");
			break;

		case AO8_QUERY_AUTO_CAL_MS:
		case AO8_QUERY_INIT_MS:
		case AO8_QUERY_SELFTEST_MS:

			gsc_label_long_comma_buf(set, buf);
			strcat(buf, " ms");
			break;

		case AO8_QUERY_FSAMP_MAX:
		case AO8_QUERY_FSAMP_MIN:

			gsc_label_long_comma_buf(set, buf);
			strcat(buf, " S/S");
			break;

		case AO8_QUERY_FIFO_SIZE:

			ptr	= "";
			tmp	= set;

			if ((tmp) && ((tmp % 1024) == 0))
			{
				ptr	= "K";
				tmp	/= 1024;
			}

			gsc_label_long_comma_buf(tmp, buf);
			strcat(buf, ptr);
			strcat(buf, " Words");
			break;

		case AO8_QUERY_DEVICE_TYPE:

			if (set == GSC_DEV_TYPE_18AO8)
			{
				strcpy(buf, "18AO8");
			}
			else
			{
				errs++;
				sprintf(buf, "INVALID: %ld", (long) set);
			}

			break;

		case AO8_QUERY_CUSTOM_FEAT:

			switch (set)
			{
				default:

					errs++;
					sprintf(buf, "Invalid option: %ld", (long) set);
					break;

				case 0:	strcpy(buf, "None");	break;
			}

			break;
	}

	if (verbose)
	{
		if (errs)
			printf("FAIL <---  (%s)\n", buf);
		else
			printf("%s\n", buf);
	}

	if (get)
		get[0]	= set;

	return(errs);
}


