// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/18AO8/18AO8_Linux_1.x_and_2.x_DN/utils/util_reg.c $
// $Rev: 52859 $
// $Date: 2023-04-17 14:38:13 -0500 (Mon, 17 Apr 2023) $

// 18AO8: Utilities: source file

#include "main.h"



// macros *********************************************************************

#define	_GSC_REG(a)				"GSC " #a, AO8_GSC_##a
#define	SHOW_FIELD(b,e,eol,l,d)	gsc_reg_field_show(width+10,23, value,(b),(e),(eol),(l),(d))
#define	SIZEOF_ARRAY(a)			(sizeof((a)) / sizeof((a)[0]))



//*****************************************************************************
static int _bcr_decode(int fd, int supported, u32 value, int width)
{
	static const char* const	disable[]	= { "Disable",			"Enable"			};
	static const char* const	fail[]		= { "Fail",				"Pass"				};
	static const char* const	idle[]		= { "Idle",				"Active"			};
	static const char* const	immed[]		= { "No, Immediate",	"Yes, Simultaneous"	};
	static const char* const	no[]		= { "No",				"Yes"				};
	static const char* const	ttl[]		= { "TTL",				"LVDS"				};
	static const char* const	twos[]		= { "2s Compliment",	"Offset Binary"		};

	SHOW_FIELD( 0,  0, 1, idle,		"Initiate Selftest"		);
	SHOW_FIELD( 1,  1, 1, no,		"Selftest Data Ready"	);
	SHOW_FIELD( 2,  2, 1, idle,		"Selftest Buffer Clear"	);
	SHOW_FIELD( 4,  3, 1, NULL,		"Reserved"				);
	SHOW_FIELD( 5,  5, 1, ttl,		"Transceiver Type"		);
	SHOW_FIELD(17,  6, 1, NULL,		"Reserved"				);
	SHOW_FIELD(18, 18, 1, immed,	"Simultaneous Outputs"	);
	SHOW_FIELD(19, 19, 1, NULL,		"Reserved"				);
	SHOW_FIELD(20, 20, 1, idle,		"Software Clock"		);
	SHOW_FIELD(21, 21, 1, disable,	"Rate-C Generator"		);
	SHOW_FIELD(22, 22, 1, disable,	"Rate-D Generator"		);
	SHOW_FIELD(23, 23, 1, idle,		"Software Trigger"		);
	SHOW_FIELD(24, 24, 1, NULL,		"Reserved"				);
	SHOW_FIELD(25, 25, 1, twos,		"Data Format"			);
	SHOW_FIELD(27, 26, 1, NULL,		"Reserved"				);
	SHOW_FIELD(28, 28, 1, idle,		"Auto-Calibration"		);
	SHOW_FIELD(29, 29, 1, fail,		"Auto-Cal. Status"		);
	SHOW_FIELD(30, 30, 1, NULL,		"Reserved"				);
	SHOW_FIELD(31, 31, 1, idle,		"Initialize"			);
	return(0);
}



//*****************************************************************************
static int _diopr_decode(int fd, int supported, u32 value, int width)
{
	static const char* const	input[]	= { "Input",	"Output"	};

	SHOW_FIELD( 3,  0, 1, NULL,		"DIO D0-D3"				);
	SHOW_FIELD( 6,  4, 1, NULL,		"Reserved"				);
	SHOW_FIELD( 7,  7, 1, input,	"DIO D0-D3 Direction"	);
	SHOW_FIELD(11,  8, 1, NULL,		"DIO D4-D7"				);
	SHOW_FIELD(14, 12, 1, NULL,		"Reserved"				);
	SHOW_FIELD(15, 15, 1, input,	"DIO D4-D7 Direction"	);
	SHOW_FIELD(31, 16, 1, NULL,		"Reserved"				);
	return(0);
}



//*****************************************************************************
static int _psr_decode(int fd, int supported, u32 value, int width)
{
	static const char* const	disable[]	= { "Disable",	"Enable"	};
	static const char* const	idle[]		= { "Idle",		"Active"	};

	SHOW_FIELD( 0,  0, 1, disable,	"Auto-Calibration"		);
	SHOW_FIELD( 4,  1, 1, NULL,		"Reserved"				);
	SHOW_FIELD( 5,  5, 1, disable,	"Selftest Data Ready"	);
	SHOW_FIELD( 7,  6, 1, NULL,		"Reserved"				);
	SHOW_FIELD( 8,  8, 1, disable,	"DIO D0 Low to High"	);
	SHOW_FIELD( 9,  9, 1, disable,	"Threshold Hi to Low"	);
	SHOW_FIELD(10, 10, 1, disable,	"Threshold Low to Hi"	);
	SHOW_FIELD(11, 11, 1, disable,	"Load Ready Hi to Low"	);
	SHOW_FIELD(12, 12, 1, disable,	"Load Ready Low to Hi"	);
	SHOW_FIELD(13, 13, 1, disable,	"Burst Ready"			);
	SHOW_FIELD(14, 14, 1, disable,	"Data/Frame Overflow"	);
	SHOW_FIELD(15, 15, 1, NULL,		"Reserved"				);

	SHOW_FIELD(16, 16, 1, idle,		"Auto-Calibration"		);
	SHOW_FIELD(20, 17, 1, NULL,		"Reserved"				);
	SHOW_FIELD(21, 21, 1, idle,		"Selftest Data Ready"	);
	SHOW_FIELD(23, 22, 1, NULL,		"Reserved"				);
	SHOW_FIELD(24, 24, 1, idle,		"DIO D0 Low to High"	);
	SHOW_FIELD(25, 25, 1, idle,		"Threshold Hi to Low"	);
	SHOW_FIELD(26, 26, 1, idle,		"Threshold Low to Hi"	);
	SHOW_FIELD(27, 27, 1, idle,		"Load Ready Hi to Low"	);
	SHOW_FIELD(28, 28, 1, idle,		"Load Ready Low to Hi"	);
	SHOW_FIELD(29, 29, 1, idle,		"Burst Ready"			);
	SHOW_FIELD(30, 30, 1, idle,		"Data/Frame Overflow"	);
	SHOW_FIELD(31, 31, 1, NULL,		"Reserved"				);
	return(0);
}



//*****************************************************************************
static int _acr_decode(int fd, int supported, u32 value, int width)
{
	static const char* const	clock[]	=
	{
		"40.320MHz",
		"RESERVED",
		"RESERVED",
		"RESERVED"
	};

	static const char* const	features[]	=
	{
		"None",
		"RESERVED",
		"RESERVED",
		"RESERVED"
	};

	static const char* const	channels[]	= { "8", "4" };

	long	v;

	SHOW_FIELD(11,  0, 0, NULL,		"Firmware Version"	);
	v	= GSC_FIELD_DECODE(value, 11, 0);
	printf("%03lX\n", v);

	SHOW_FIELD(15, 12, 1, NULL,		"Reserved"			);
	SHOW_FIELD(16, 16, 1, channels,	"Output Channels"	);
	SHOW_FIELD(18, 17, 1, clock,	"Master Clock"		);
	SHOW_FIELD(20, 19, 1, features,	"Custom Features"	);
	SHOW_FIELD(31, 21, 1, NULL,		"Reserved"			);
	return(0);
}



//*****************************************************************************
static int _boor_decode(int fd, int supported, u32 value, int width)
{
	static const char* const	clear[]		= { "Clear",		"Set"		};
	static const char* const	disable[]	= { "Disabled",		"Enabled"	};
	static const char* const	idle[]		= { "Idle",			"Active"	};
	static const char* const	circ[]		= { "Open",			"Circular"	};
	static const char* const	no[]		= { "No",			"Yes"		};

	SHOW_FIELD( 4,  0, 1, NULL,		"Reserved"			);
	SHOW_FIELD( 5,  5, 1, disable,	"Output Clocking"	);
	SHOW_FIELD( 6,  6, 1, no,		"Clock Ready"		);
	SHOW_FIELD( 7,  7, 1, idle,		"SW Clock"			);
	SHOW_FIELD( 8,  8, 1, circ,		"Buffer Mode"		);
	SHOW_FIELD( 9,  9, 1, idle,		"Load Request"		);
	SHOW_FIELD(10, 10, 1, no,		"Load Ready"		);
	SHOW_FIELD(11, 11, 1, idle,		"Clear Buffer"		);
	SHOW_FIELD(12, 12, 1, no,		"Buffer Empty"		);
	SHOW_FIELD(13, 13, 1, clear,	"Buffer Threshold"	);
	SHOW_FIELD(14, 14, 1, NULL,		"Reserved"			);
	SHOW_FIELD(15, 15, 1, no,		"Buffer Full"		);
	SHOW_FIELD(16, 16, 1, no,		"Buffer Overflow"	);
	SHOW_FIELD(17, 17, 1, no,		"Frame Overflow"	);
	SHOW_FIELD(18, 18, 1, no,		"Burst Ready"		);
	SHOW_FIELD(19, 19, 1, disable,	"Output Bursting"	);
	SHOW_FIELD(20, 20, 1, idle,		"SW Trigger"		);
	SHOW_FIELD(31, 21, 1, NULL,		"Reserved"			);
	return(0);
}



//*****************************************************************************
static int _obtr_decode(int fd, int supported, u32 value, int width)
{
	static const char* const	clear[]	= { "Clear",	"Set"	};

	SHOW_FIELD(18,  0, 1, NULL,		"Threshold"			);
	SHOW_FIELD(19, 19, 1, NULL,		"Reserved"			);
	SHOW_FIELD(20, 20, 1, clear,	"Threshold Flag"	);
	SHOW_FIELD(31, 21, 1, NULL,		"Reserved"			);
	return(0);
}



//*****************************************************************************
static int _obsr_decode(int fd, int supported, u32 value, int width)
{
	SHOW_FIELD(18,  0, 1, NULL,	"Fill Level"	);
	SHOW_FIELD(31, 19, 1, NULL,	"Reserved"		);
	return(0);
}



//*****************************************************************************
static int _rxgr_decode(int fd, int supported, u32 value, int width)
{
	SHOW_FIELD(23,  0, 1, NULL,	"Ndiv"		);
	SHOW_FIELD(31, 24, 1, NULL,	"Reserved"	);
	return(0);
}



//*****************************************************************************
static int _ocr_decode(int fd, int supported, u32 value, int width)
{
	static const char* const	range[]	=
	{
		"+-2.5 Volts",
		"+-5 Volts",
		"+-10 Volts",
		"RESERVED",
		"RESERVED",
		"RESERVED",
		"RESERVED",
		"RESERVED"
	};

	static const char* const	clk_src[]	=
	{
		"Rate-C Generator",
		"RESERVED",
		"External Input",
		"RESERVED"
	};

	static const char* const	trg_src[]	=
	{
		"Rate-D Generator",
		"RESERVED",
		"External Input",
		"RESERVED"
	};

	static const char* const	defalt[]	= { "Default",		"Invert"		};
	static const char* const	disable[]	= { "Disable",		"Enable"		};
	static const char* const	se_diff[]	= { "Single Ended",	"Differential"	};

	s32	channels	= 8;
	int	errs;

	errs	= ao8_query(fd, -1, 0, AO8_QUERY_CHANNEL_QTY, &channels);

	SHOW_FIELD		( 0,  0, 1, disable,	"Output 0"				);
	SHOW_FIELD		( 1,  1, 1, disable,	"Output 1"				);
	SHOW_FIELD		( 2,  2, 1, disable,	"Output 2"				);
	SHOW_FIELD		( 3,  3, 1, disable,	"Output 3"				);

	if (channels > 4)
	{
		SHOW_FIELD	( 4,  4, 1, disable,	"Output 4"				);
		SHOW_FIELD	( 5,  5, 1, disable,	"Output 5"				);
		SHOW_FIELD	( 6,  6, 1, disable,	"Output 6"				);
		SHOW_FIELD	( 7,  7, 1, disable,	"Output 7"				);
	}
	else
	{
		SHOW_FIELD	( 7,  4, 1, NULL,		"Reserved"				);
	}

	SHOW_FIELD		(10,  8, 1, range,		"Voltage Range"			);
	SHOW_FIELD		(11, 11, 1, disable,	"Clock/Sync Outputs"	);
	SHOW_FIELD		(13, 12, 1, clk_src,	"Clock Source"			);
	SHOW_FIELD		(15, 14, 1, trg_src,	"Trigger Source"		);
	SHOW_FIELD		(16, 16, 1, se_diff,	"Output Configuration"	);
	SHOW_FIELD		(17, 17, 1, defalt,		"Transceiver Polarity"	);
	SHOW_FIELD		(18, 18, 1, NULL,		"Maintenance Utility"	);
	SHOW_FIELD		(31, 19, 1, NULL,		"Reserved"				);
	return(errs);
}



//*****************************************************************************
static int _scr_decode(int fd, int supported, u32 value, int width)
{
	static const char* const	mode[]	=
	{
		"Zero Volts Self Test",
		"+Vref Self Test",
		"RESERVED",
		"RESERVED",
		"RESERVED",
		"RESERVED",
		"RESERVED",
		"RESERVED",
		"Output Channel 0",
		"Output Channel 1",
		"Output Channel 2",
		"Output Channel 3",
		"Output Channel 4",
		"Output Channel 5",
		"Output Channel 6",
		"Output Channel 7"
	};

	SHOW_FIELD( 3,  0, 1, mode,	"Measurement Mode"	);
	SHOW_FIELD(31,  4, 1, NULL,	"Reserved"			);
	return(0);
}



//*****************************************************************************
static int _sdbr_decode(int fd, int supported, u32 value, int width)
{
	SHOW_FIELD(17,  0, 1, NULL,	"Data"		);
	SHOW_FIELD(31, 18, 1, NULL,	"Reserved"	);
	return(0);
}



//*****************************************************************************
static int _asiocr_decode(int fd, int supported, u32 value, int width)
{
	static const char* const	mode[]	=
	{
		"Inactive",
		"Inout",
		"Output",
		"RESERVED"
	};

	static const char* const	inv_in[]	= { "No, Low-to-High Edge",		"Yes, High-to-Low Edge"		};
	static const char* const	inv_out[]	= { "No, Low-to-High Pulse",	"Yes, High-to-Low Pulse"	};
	static const char* const	noise[]		= { "Low, 90-120ns, 1.3us",		"High, 135ns, 1.8us"		};

	SHOW_FIELD( 1,  0, 1, mode,		"Aux Clock Mode"	);
	SHOW_FIELD( 3,  2, 1, mode,		"Aux Sync Mode"		);
	SHOW_FIELD( 7,  4, 1, NULL,		"Reserved"			);
	SHOW_FIELD( 8,  8, 1, inv_in,	"Invert Inputs"		);
	SHOW_FIELD( 9,  9, 1, inv_out,	"Invert Outputs"	);
	SHOW_FIELD(10, 10, 1, noise,	"Noise Supression"	);
	SHOW_FIELD(31, 11, 1, NULL,		"Reserved"			);
	return(0);
}



// variables ******************************************************************

static gsc_reg_def_t	_gsc[]	=
{
	// name	reg			err	value	ask_support	decode				desc
	{ _GSC_REG(BCR),	0,	0,		0,			_bcr_decode,		"Board Control Register"				},
	{ _GSC_REG(DIOPR),	0,	0,		0,			_diopr_decode,		"Digital I/O Port Register"				},
	{ _GSC_REG(SCR),	0,	0,		0,			_scr_decode,		"Selftest Cofiguration Register"		},
	{ _GSC_REG(SDBR),	0,	0,		0,			_sdbr_decode,		"Selftest Data Buffer Register"			},
	{ _GSC_REG(ASIOCR),	0,	0,		0,			_asiocr_decode,		"Auxiliary Sync I/O Register"			},
	{ _GSC_REG(PSR),	0,	0,		0,			_psr_decode,		"Primary Status Register"				},
	{ _GSC_REG(ACR),	0,	0,		0,			_acr_decode,		"Assembly Configuration Register"		},
	{ _GSC_REG(AVR),	0,	0,		0,			NULL,				"Auto Cal Values Register"				},
	{ _GSC_REG(BOOR),	0,	0,		0,			_boor_decode,		"Buffered Output Operations Register"	},
	{ _GSC_REG(OBTR),	0,	0,		0,			_obtr_decode,		"Output Buffer Threshold Register"		},
	{ _GSC_REG(OBSR),	0,	0,		0,			_obsr_decode,		"Output Buffer Size Register"			},
	{ _GSC_REG(AOBR),	0,	0,		0,			NULL,				"Analog Output Buffer Register"			},
	{ _GSC_REG(RCGR),	0,	0,		0,			_rxgr_decode,		"Rate-C Generator Register"				},
	{ _GSC_REG(RDGR),	0,	0,		0,			_rxgr_decode,		"Rate-D Generator Register"				},
	{ _GSC_REG(OCR),	0,	0,		0,			_ocr_decode,		"Output Configuration Register"			},
	{ NULL }
};



//*****************************************************************************
static const gsc_reg_def_t* _find_reg(u32 reg, const gsc_reg_def_t* list)
{
	const gsc_reg_def_t*	def	= NULL;
	int						i;

	for (i = 0; list[i].name; i++)
	{
		if (reg == list[i].reg)
		{
			def	= &list[i];
			break;
		}
	}

	return(def);
}



/******************************************************************************
*
*	Function:	ao8_reg_get_def_id
*
*	Purpose:
*
*		Retrieve the register definition structure given the register id.
*
*	Arguments:
*
*		reg		The id of the register to access.
*
*	Returned:
*
*		NULL	The register id wasn't found.
*		else	A pointer to the register definition.
*
******************************************************************************/

const gsc_reg_def_t* ao8_reg_get_def_id(u32 reg)
{
	const gsc_reg_def_t*	def;

	def	= _find_reg(reg, _gsc);
	return(def);
}



/******************************************************************************
*
*	Function:	ao8_reg_get_def_index
*
*	Purpose:
*
*		Retrieve the register definition structure based on an index.
*
*	Arguments:
*
*		index	The index of the register to access.
*
*	Returned:
*
*		NULL	The index doesn't correspond to a known register.
*		else	A pointer to the register definition.
*
******************************************************************************/

const gsc_reg_def_t* ao8_reg_get_def_index(int index)
{
	const gsc_reg_def_t*	def;

	if (index < 0)
		def	= NULL;
	else if (index >= (SIZEOF_ARRAY(_gsc) - 1))
		def	= NULL;
	else
		def	= &_gsc[index];

	return(def);
}



/******************************************************************************
*
*	Function:	ao8_reg_get_desc
*
*	Purpose:
*
*		Retrieve the description of the specified register.
*
*	Arguments:
*
*		reg		The register whose description is desired.
*
*	Returned:
*
*		!NULL	The register's name.
*
******************************************************************************/

const char* ao8_reg_get_desc(u32 reg)
{
	const gsc_reg_def_t*	def;
	const char*				desc;

	def	= _find_reg(reg, _gsc);

	if (def)
		desc	= def->desc;
	else
		desc	= "UNKNOWN";

	return(desc);
}



/******************************************************************************
*
*	Function:	ao8_reg_get_name
*
*	Purpose:
*
*		Retrieve the name of the specified register.
*
*	Arguments:
*
*		reg		The register whose name is desired.
*
*	Returned:
*
*		!NULL	The register's name.
*
******************************************************************************/

const char* ao8_reg_get_name(u32 reg)
{
	const gsc_reg_def_t*	def;
	const char*				name;

	def	= _find_reg(reg, _gsc);

	if (def)
		name	= def->name;
	else
		name	= "UNKNOWN";

	return(name);
}



/******************************************************************************
*
*	Function:	ao8_reg_list
*
*	Purpose:
*
*		List the GSC registers and their values.
*
*	Arguments:
*
*		fd		The handle to access the device.
*
*		detail	List the register details?
*
*	Returned:
*
*		>= 0	The number of errors encountered here.
*
******************************************************************************/

int ao8_reg_list(int fd, int detail)
{
	int	errs;

	errs	= gsc_reg_list(fd, _gsc, detail, ao8_reg_read);
	return(errs);
}



