// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/18AO8/18AO8_Linux_1.x_and_2.x_DN/utils/util_selftest_mode.c $
// $Rev: 52859 $
// $Date: 2023-04-17 14:38:13 -0500 (Mon, 17 Apr 2023) $

// 18AO8: Utilities: source file

#include "main.h"



/******************************************************************************
*
*	Function:	ao8_selftest_mode
*
*	Purpose:
*
*		Provide a visual wrapper for the AO8_IOCTL_SELFTEST_MODE service.
*
*	Arguments:
*
*		fd		Use this handle to access the device.
*
*		index	The index of the device to access. Ignore if < 0.
*				This is for display purposes only.
*
*		verbose	Work verbosely?
*
*		set		This is the value to apply.
*
*		get		The results are reported here. This may be NULL.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int ao8_selftest_mode(int fd, int index, int verbose, s32 set, s32* get)
{
	char		buf[128];
	int			errs;
	const char*	ptr;
	int			ret;

	if (verbose)
		gsc_label_index("Self Test Mode", index);

	ret		= ao8_ioctl(fd, AO8_IOCTL_SELFTEST_MODE, &set);
	errs	= ret ? 1 : 0;

	switch (set)
	{
		default:

			errs++;
			ptr	= buf;
			sprintf(buf, "Unrecognized option: 0x%lX", (long) set);
			break;

		case AO8_SELFTEST_MODE_ZERO:

			ptr	= "Zero Voltage";
			break;

		case AO8_SELFTEST_MODE_VREF:

			ptr	= "Vref Test";
			break;

		case AO8_SELFTEST_MODE_OUT_0:

			ptr	= "Channel 0 Output";
			break;

		case AO8_SELFTEST_MODE_OUT_1:

			ptr	= "Channel 1 Output";
			break;

		case AO8_SELFTEST_MODE_OUT_2:

			ptr	= "Channel 2 Output";
			break;

		case AO8_SELFTEST_MODE_OUT_3:

			ptr	= "Channel 3 Output";
			break;

		case AO8_SELFTEST_MODE_OUT_4:

			ptr	= "Channel 4 Output";
			break;

		case AO8_SELFTEST_MODE_OUT_5:

			ptr	= "Channel 5 Output";
			break;

		case AO8_SELFTEST_MODE_OUT_6:

			ptr	= "Channel 6 Output";
			break;

		case AO8_SELFTEST_MODE_OUT_7:

			ptr	= "Channel 7 Output";
			break;
	}

	if (verbose)
		printf("%s  (%s)\n", errs ? "FAIL <---" : "PASS", ptr);

	if (get)
		get[0]	= set;

	return(errs);
}


